@@ -1,7 +1,7 @@
 import threading
 import shlex
 
-import cloudlink
+from . import cloudlink
 import sys
 
 import json
@@ -16,6 +16,8 @@ from .command import AppCommand
 from .context import CTX
 
 import time
+import logging
+
 
 class Bot:
     """
@@ -31,17 +33,13 @@ class Bot:
     ]
 
     def _t_ping(self):
-       while True:
-         time.sleep(300)
+        while True:
+            time.sleep(60)
 
-         self.wss.sendPacket({"cmd":"ping", "val":""})
-
-    def __init__(self, prefix=None, debug=False, debug_out=sys.__stdout__):
-        self.wss = cloudlink.CloudLink(debug=debug)
-        self._stdout = sys.__stdout__
-        self.debug = debug
-        self.debug_out = debug_out
+            self.wss.sendPacket({"cmd": "ping", "val": ""})
 
+    def __init__(self, prefix=None):
+        self.wss = cloudlink.CloudLink()
         self.callbacks = {}
 
         self.wss.callback(
@@ -56,11 +54,15 @@ class Bot:
         # to be used in start
         self.username = None
         self.password = None
-        self.logging_in = False
+        self.logger_in = False
 
         self.commands = {}
-        self.prefix = prefix 
-        self._t_ping_thread = threading.Thread(target=self._t_ping, daemon=True) # (:
+        self.prefix = prefix
+        self._t_ping_thread = threading.Thread(target=self._t_ping, daemon=True)  # (:
+        self.logger = logging.getLogger("MeowerBot")
+        self.bad_exit = False
+
+        self.cogs = {}
 
     def run_cb(self, cbid, args=(), kwargs=None):  # cq: ignore
         if cbid not in self.callbacks:
@@ -70,16 +72,15 @@ class Bot:
             kwargs = {}
 
         kwargs["bot"] = self
-        print(kwargs, "\n", args)
         for callback in self.callbacks[cbid]:
             try:
                 callback(
-                	*args, **kwargs
+                    *args, **kwargs
                 )  # multi callback per id is supported (unlike cloudlink 0.1.7.3 LOL)
-            except Exception as e: #cq ignore
-               if self.debug:
-                  print(traceback.format_exc(), file=self.debug_out)
-               self.run_cb("error", args=(e))
+            except Exception as e:  # cq ignore
+
+                self.logger.error(traceback.format_exc())
+                self.run_cb("error", args=(e))
 
     def __handle_error__(self, e):
         self.run_cb("error", args=(e))
@@ -90,15 +91,14 @@ class Bot:
         try:
             self.__handle_packet__(packet)
         except Exception as e:  # cq: skip #IDC ABOUT GENERAL EXCP
-            if self.debug:
-                print(traceback.format_exc() ,file=self.debug_out)
+
+            self.logger.error(traceback.format_exc())
             self.run_cb("error", args=(e))
 
         try:
             self.run_cb("__raw__", args=(packet))  # raw packets
         except Exception as e:  # cq: skip #IDC ABOUT GENERAL EXCP
-            if self.debug:
-                print(traceback.format_exc(), file=self.debug_out)
+            self.logger.error(traceback.format_exc())
             self.run_cb("error", args=(e))
 
     def __handle_on_connect__(self):
@@ -114,47 +114,52 @@ class Bot:
         )
 
         self.wss.sendPacket(
-                {
-                    "cmd": "direct",
-                    "val": {"cmd": "type", "val": "py"},
-                }
-            )
+            {
+                "cmd": "direct",
+                "val": {"cmd": "type", "val": "py"},
+            }
+        )
         self.wss.sendPacket(
-                {
-                    "cmd": "direct",
-                    "val": "meower",
-                    "listener": "__meowerbot__cloudlink_trust",
-                }
-            )
+            {
+                "cmd": "direct",
+                "val": "meower",
+                "listener": "__meowerbot__cloudlink_trust",
+            }
+        )
 
-    def command(self, aname = None, args = 0):
+    def command(self, aname=None, args=0):
         def inner(func):
             if aname is None:
                 name = func.__name__
             else:
                 name = aname
 
-
-            cmd = AppCommand(func, name=name, args = args)
+            cmd = AppCommand(func, name=name, args=args)
 
             info = cmd.info()
-            info[cmd.name]['command'] = cmd
-
+            info[cmd.name]["command"] = cmd
 
             self.commands.update(info)
 
             return func
+
         return inner
 
     def register_cog(self, cog):
         info = cog.get_info()
+        self.cogs[cog.__class__.__name__] = cog
         self.commands.update(info)
 
+    def deregister_cog(self, cogname):
+        for cmd in self.cogs[cogname].get_info().values():
+            del self.commands[cmd.name]
+        del self.cogs[cogname]
 
     def _handle_status(self, status, listener):
-        if status == "I:112 | Trusted Access enabled": return 
-        if self.logging_in:
-            self.logging_in = False
+        if status == "I:112 | Trusted Access enabled":
+            return
+        if self.logger_in:
+            self.logger_in = False
             if not status == "I:100 | OK":
                 raise RuntimeError("CloudLink Trust Failed")
 
@@ -163,17 +168,29 @@ class Bot:
                     "cmd": "direct",
                     "val": {
                         "cmd": "authpswd",
-                        "val":{"username":self.username, "pswd":self._password}
+                        "val": {"username": self.username, "pswd": self._password},
                     },
                     "listener": "__meowerbot__login",
                 }
             )
 
         elif listener == "__meowerbot__login":
+            if status == "E:104 | Internal":
+                requests.post(
+                    "https://webhooks.meower.org/post/home",
+                    json={
+                        "post": "ERROR: MeowerBot.py Webhooks Logging\n\n Account Softlocked.",
+                        "username": self.username,
+                    },
+                )
+                print("CRITICAL ERROR! ACCOUNT SOFTLOCKED!!!!.", file=sys.__stdout__)
+                self.bad_exit = True
+                self.wss.stop()
+                return
+
             if not status == "I:100 | OK":
                 raise RuntimeError("Password Or Username Is Incorrect")
 
-
             time.sleep(0.5)
             self.run_cb("login", args=(), kwargs={})
 
@@ -185,7 +202,8 @@ class Bot:
 
     def callback(self, callback, cbid=None):
         """Connects a callback ID to a callback"""
-        if cbid is None: cbid = callback.__name__
+        if cbid is None:
+            cbid = callback.__name__
 
         if cbid not in self.callbacks:
             self.callbacks[cbid] = []
@@ -196,36 +214,35 @@ class Bot:
 
     def __handle_packet__(self, packet):
         if packet["cmd"] == "statuscode":
-  
-                self._handle_status(packet["val"], packet.get("listener", None))
 
-                listener = packet.get("listener", None)
-                return self.run_cb("statuscode", args=(packet["val"], listener))
+            self._handle_status(packet["val"], packet.get("listener", None))
+
+            listener = packet.get("listener", None)
+            return self.run_cb("statuscode", args=(packet["val"], listener))
 
         elif packet["cmd"] == "ulist":
             self.run_cb("ulist", self.wss.statedata["ulist"]["usernames"])
 
-        elif (
-            packet["cmd"] == "direct" and "post_origin" in packet["val"]
-        ):
-            if packet['val']['u'] == "Discord" and ": " in packet['val']['p']:
-                split = packet['val']['p'].split(": ")
-                packet['val']['p'] = split[1]
-                packet['val']['u'] = split[0]
+        elif packet["cmd"] == "direct" and "post_origin" in packet["val"]:
+            if packet["val"]["u"] == "Discord" and ": " in packet["val"]["p"]:
+                split = packet["val"]["p"].split(": ")
+                packet["val"]["p"] = split[1]
+                packet["val"]["u"] = split[0]
 
-
-            ctx = CTX(packet['val'], self)
+            ctx = CTX(packet["val"], self)
             if "message" in self.callbacks:
-               self.run_cb('message', args = (ctx.message,) )
+                self.run_cb("message", args=(ctx.message,))
 
             else:
 
-               if ctx.user.username == self.username: return
-               if not ctx.message.data.startswith(self.prefix): return
+                if ctx.user.username == self.username:
+                    return
+                if not ctx.message.data.startswith(self.prefix):
+                    return
 
-               ctx.message.data = ctx.message.data.split(self.prefix, 1)[1]
+                ctx.message.data = ctx.message.data.split(self.prefix, 1)[1]
 
-               self.run_command(ctx.message)
+                self.run_command(ctx.message)
 
             self.run_cb("raw_message", args=(packet["val"],))
 
@@ -238,38 +255,87 @@ class Bot:
             self.run_cb(packet["cmd"], args=(packet["val"], listener))
 
     def run_command(self, message):
-      args = shlex.split(str(message))
+        args = shlex.split(str(message))
 
-      try:
-        self.commands[args[0]]['command'].run_cmd(message.ctx, *args[1:])
-      except KeyError as e:
-        self.run_cb("error", args=(e,))
+        try:
+            self.commands[args[0]]["command"].run_cmd(message.ctx, *args[1:])
+        except KeyError as e:
+            self.run_cb("error", args=(e,))
 
     def send_msg(self, msg, to="home"):
         if to == "home":
             self.wss.sendPacket(
-                {"cmd": "direct", "val": {"cmd": "post_home", "val": msg}, "listener":"__meowerbot__send_message"}
+                {
+                    "cmd": "direct",
+                    "val": {"cmd": "post_home", "val": msg},
+                    "listener": "__meowerbot__send_message",
+                }
             )
         else:
             self.wss.sendPacket(
                 {
                     "cmd": "direct",
                     "val": {"cmd": "post_chat", "val": {"chatid": to, "p": msg}},
-                    "listener":"__meowerbot__send_message"
+                    "listener": "__meowerbot__send_message",
                 }
             )
 
+    def send_typing(self, to="home"):
+        if  to == "home":
+            self.wss.sendPacket(
+                {
+                    "cmd": "direct",
+                    "val": {
+                        "cmd": "set_chat_state",
+                        "val": {
+                            "chatid": "livechat",
+                            "state": 101,
+                        },
+                    },
+                }
+            )
+        else:
+          self.wss.sendPacket(
+            {
+                "cmd": "direct",
+                "val": {
+                    "cmd": "set_chat_state",
+                    "val": {
+                        "chatid": to,
+                        "state": 100,
+                    },
+                },
+            }
+          )
+       
+    def enter_chat(self, chatid="livechat"):
+        self.wss.sendPacket(
+            {
+                "cmd": "direct",
+                "val": {
+                    "cmd": "set_chat_state",
+                    "val": {
+                        "chatid": chatid,
+                        "state": 1,
+                    },
+                },
+            }
+        )
+
+
     def run(self, username, password, server="wss://server.meower.org"):
         """
         Runs The bot (Blocking)
         """
         self.username = username
         self._password = password
-        self.logging_in = True
- 
+        self.logger_in = True
 
         self._t_ping_thread.start()
-        if self.prefix is None: self.prefix = "@" + self.username
-        with self.debug_out as sys.stdout:
-            self.wss.client(server)
+        if self.prefix is None:
+            self.prefix = "@" + self.username
+        self.logger = logging.getLogger(f"MeowerBot {self.username}")
+        self.wss.client(server)
 
+        if self.bad_exit:
+            raise BaseException("Bot Account Softlocked")
